<?php
	class Server {
		// Klassen
		private $sql      = NULL;
		private $settings = NULL;
		private $tpl      = NULL;
		public  $rcon     = NULL;
		
		// Server-Eigenschaften
		public  $serverid = NULL;
		public  $id       = NULL;
		public  $name     = NULL;
		public  $desc     = NULL;
		public  $game     = NULL;
		public  $platform = NULL;
		public  $ip       = NULL;
		public  $port     = NULL;
		public  $password = NULL;
		public  $fetch    = NULL;
		
		// Info-Arrays
		public  $urls      = NULL;
		public  $mapnames  = NULL;
		public  $gamemodes = NULL;
		public  $squads    = NULL;
		
		// Daten Server-Info
		public  $error    = NULL;
		public  $message  = NULL;
		public  $info     = NULL;
		
		// Daten Server-Live-Infos
		public  $online   = FALSE;
		public  $player   = Array();
		public  $round    = NULL;
		public  $rounds   = NULL;
		public  $mapname  = NULL;
		public  $maplist  = "";
		public  $playmode = NULL;
		public  $score    = NULL;
		
		
		public function __construct() {
			$this->sql      = new MySQL();
			$this->settings = new Settings();
			$this->tpl      = new Templates();
			
			$this->urls = Array(
				"api" => Array(
					"bf3"  => "http://api.bf3stats.com/",
					"mohw" => "http://api.mohwstats.com/"
				),
				"bl" => Array(
					"bf3"  => "http://battlelog.battlefield.com/bf3/servers/show/",
					"mohw" => "http://battlelog.medalofhonor.com/mohw/de/servers/show/"
				),
				"url" => Array(
					"bf3"  => "http://bf3stats.com/",
					"mohw" => "http://mohwstats.com/"
				)
			);
			
			$this->mapnames = Array(
				"bf3" => Array(
					"MP_001" => "Grand Bazaar",
					"MP_003" => "Tehran Highway",
					"MP_007" => "Caspian Border",
					"MP_011" => "Seine Crossing",
					"MP_012" => "Operation Firestorm",
					"MP_013" => "Damavand Peak",
					"MP_017" => "Noshahar Canals",
					"MP_018" => "Kharg Island",
					"MP_Subway" => "Operation Metro",
					"XP1_001" => "Strike at Karkand",
					"XP1_002" => "Gulf of Oman",
					"XP1_003" => "Sharqi Peninsula",
					"XP1_004" => "Wake Island",
					"XP2_Factory" => "Scrapmetal",
					"XP2_Office" => "Operation 925",
					"XP2_Palace" => "Donya Fortress",
					"XP2_Skybar" => "Ziba Tower",
					"XP3_Desert" => "Bandar Desert",
					"XP3_Alborz" => "Alborz Mountains",
					"XP3_Shield" => "Armored Shield",
					"XP3_Valley" => "Death Valley",
					"XP4_FD" => "Markaz Monolith",
					"XP4_Parl" => "Azadi Palace",
					"XP4_Quake" => "Epicenter",
					"XP4_Rubble" => "Talah Market",
					"XP5_001" => "Operation Riverside",
					"XP5_002" => "Nebandan Flats",
					"XP5_003" => "Kiasar Railroad",
					"XP5_004" => "Sabalan Pipeline"
				),
				"mohw" => Array(
					"MP_03" => "Somalia Stronghold",
					"MP_05" => "Novi Grad Warzone",
					"MP_10" => "Sarajevo Stadium",
					"MP_12" => "Basilan Aftermath",
					"MP_13" => "Hara Dunes",
					"MP_16" => "Al Fara Cliffside",
					"MP_18" => "Shogore Valley",
					"MP_19" => "Tungawan Jungle",
					"MP_20" => "Darra Gun Market",
					"MP_21" => "Chitral Compound"
				)
			);
			
			$this->gamemodes = Array(
				"bf3" => Array(
					"1" => "Conquest",
					"2" => "Rush",
					"4" => "Squad Rush",
					"8" => "Squad Deathmatch",
					"32" => "Team Deathmatch",
					"64" => "Conquest Large",
					"128" => "Conquest Assault Large",
					"256" => "Conquest Assault",
					"512" => "Gun Master",
					"1024" => "Conquest Domination",
					"2048" => "Team Deathmatch Close Quarters",
					"131072" => "Tank Superiority",
					"524288" => "Capture the Flag",
					"4194304" => "Scavenger",
					"8388608" => "Air Superiority"
				),
				"mohw" => Array(
					"0" => "Unknown",
					"32" => "Team Deathmatch",
					"8192" => "Combat Mission",
					"16384" => "Sector Control",
					"65536" => "Home Run",
					"1048576" => "Hotspot"
				)
			);
			
			$this->squads = Array(
				"0" => "&nbsp;",
				"1" => "A",
				"2" => "B",
				"3" => "C",
				"4" => "D",
				"5" => "E",
				"6" => "F",
				"7" => "G",
				"8" => "H",
				"9" => "I",
				"10" => "J",
				"11" => "K",
				"12" => "L",
				"13" => "X",
				"31" => "X",
				"32" => "X"
			);
		}
		
		
		/* Ldt die Eigenschaften des Servers aus der Datenbank */
		public function set($id) {
			$this->sql->query('SELECT * FROM '.$this->sql->prefix.'server WHERE `id` = "'.$id.'"');
			$row = $this->sql->fetchRow();
			
			$this->id       = $id;
			$this->name     = $row['name'];
			$this->desc     = $row['description'];
			$this->serverid = $row['serverid'];
			$this->game     = $row['game'];
			$this->platform = $row['platform'];
			$this->ip       = $row['ip'];
			$this->port     = $row['port'];
			$this->password = $row['password'];
			$this->fetch    = $row;
		}
		
		
		/* Gibt die Serverinfos aus */
		public function get() {
			$this->sql->query('SELECT * FROM '.$this->sql->prefix.'cache WHERE `id` = "'.$this->id.'"');
			
			// Server ist noch nicht gecached -> API-Abruf
			if ($this->sql->count() == 0) { 
				$this->loadAPI($this->serverid, $this->platform, $this->game);
				
				// Abruf der API-Daten war erfolgreich
				if ($this->error === false) {
					$this->enhanceData();
					$this->cache("save");
				}
			}
			
			// Server ist gecached
			else {
				$now   = time();
				$cache = $this->sql->fetchRow();
				
				// Letzter Cache ist lter als in den Settings hinterlegt
				if ($this->settings->get("cache") + $cache['time'] <= $now) {
					// Daten von der API laden
					$this->loadAPI($this->serverid, $this->platform, $this->game);
					
					// Abruf der API-Daten war erfolgreich
					if ($this->error === false) {
						$this->enhanceData();
						$this->cache("save");
					}
					
					// Abruf der API-Daten war nicht erfolgreich -> Cache laden
					else {
						$this->cache("load");
						$this->enhanceData();
					}
				} 
				
				// Daten aus dem Cache laden
				else {
					$this->cache("load");
					$this->enhanceData();
				}
			}
		}
		
		
		/* Gibt die Live-Daten des Servers aus */
		public function live($admin=false) {
			if ($this->ip != "" AND $this->port != "") {
				$this->rcon = new BF3Conn($this->ip, $this->port);
				
				if ($this->rcon->isServerOnline()) {
					$this->online = true;
					
					/* LIVE ROUND-STATS */
					if (true) {
						$info = $this->rcon->getServerInfo();
						
						$this->round = Array(
							"currentRound" => $info[6] +1,
							"maxRounds"    => $info[7],
							"ticketsUS"    => round($info[9]),
							"ticketsRU"    => round($info[10]),
							"spawnsLeft"   => $info[11],
							"currPlayers"  => $info[2],
							"maxPlayers"   => $info[3],
							"all"          => $info
						);
						
						$this->rounds = ($info[6] + 1).' / '.$info[7];
					}
					
					/* LIVE SPIELER-STATS */
					if (true) {
						if ($this->game == "bf3") {
							$result = $this->rcon->getPlayerlist();
							
							// berflssige Werte entfernen
							for ($i = 0; $i < 11; $i++) {
								unset($result[$i]);
							}
							$result = array_values($result);
							
							
							$i = 0;
							// Schleife wiederholen bis $result leer ist
							while (count($result)) {
								
								// Daten fllen
								$this->player[$i] = Array();
								$this->player[$i]["name"]   = $result[0];
								$this->player[$i]["team"]   = $result[2];
								if (isset($this->squads[$result[3]])) $this->player[$i]["squad"] = $this->squads[$result[3]]; else $this->player[$i]["squad"] = "X";
								$this->player[$i]["kills"]  = $result[4];
								$this->player[$i]["deaths"] = $result[5];
								$this->player[$i]["score"]  = $result[6];
								
								//echo "<pre>"; print_r($result); die();
								// Datensatz lschen
								for ($j = 0; $j < 8; $j++) {
									unset($result[$j]);
								}
								$result = array_values($result);
								$i++;
							}
						} 
						else if ($this->game == "mohw") {
							$result = $this->rcon->getPlayerlist();
							
							// berflssige Werte entfernen
							for ($i = 0; $i < 10; $i++) {
								unset($result[$i]);
							}
							$result = array_values($result);
							
							
							$i = 0;
							// Schleife wiederholen bis $result leer ist
							while (count($result)) {
								
								// Daten fllen
								$this->player[$i] = Array();
								$this->player[$i]["name"]   = $result[0];
								$this->player[$i]["team"]   = $result[2];
								if (isset($this->squads[$result[3]])) $this->player[$i]["squad"] = $this->squads[$result[3]]; else $this->player[$i]["squad"] = "X";
								$this->player[$i]["kills"]  = $result[4];
								$this->player[$i]["deaths"] = $result[5];
								$this->player[$i]["score"]  = $result[6];
								
								// Datensatz lschen
								for ($j = 0; $j < 7; $j++) {
									unset($result[$j]);
								}
								$result = array_values($result);
								$i++;
							}
						} 
						else {
							echo "Error: This game is not supported";
						}
					}
					
					
					/* MAPLIST */
					if ($admin AND $this->password != "") {
						$this->rcon->loginSecure($this->password);
						
						if ($this->rcon->isLoggedIn()) {
							$result = $this->rcon->adminMaplistList(); // Mapliste laden (Array). 
							//echo "<pre>"; print_r($result);
							
							$test1 = substr($result[3], 0, 2);
							$test2 = substr($result[4], 0, 2);
							
							if ($test1 == "MP" OR $test1 == "XP") {
								$result = array_slice($result, 3); // Drei nutzlose Eintrge am Beginn entfernen
								$count = count($result) / 3; // Drei Datenstze gehren zu einer Map. Daher Counter durch 3 nehmen
								
								for ($i = 0; $i < $count; $i++) {
									$map = array_slice($result, 0, 3); // Datensatz einer Map extrahieren
									$result = array_slice($result, 3); // Restliche Datenstze zurckschreiben
									$this->maplist .= '<option value="'.$i.'">'.$this->mapnames[$this->game][$map[0]].' ('.$map[0].') - '.$map[1].'</option>';
								}
							} 
							else if ($test2 == "MP" OR $test2 == "XP") {
								$result = array_slice($result, 4); // Vier nutzlose Eintrge am Beginn entfernen
								$count = count($result) / 3; // Drei Datenstze gehren zu einer Map. Daher Counter durch 3 nehmen
								
								for ($i = 0; $i < $count; $i++) {
									$map = array_slice($result, 0, 3); // Datensatz einer Map extrahieren
									$result = array_slice($result, 3); // Restliche Datenstze zurckschreiben
									$this->maplist .= '<option value="'.$i.'">'.$this->mapnames[$this->game][$map[0]].' ('.$map[0].') - '.$map[1].'</option>';
								}
							} 
							else {
								$result = array_slice($result, 1); // Ein nutzloser Eintrag am Beginn entfernen
								$count = count($result) / 2; // Zwei Datenstze gehren zu einer Map. Daher Counter durch 2 nehmen
								
								for ($i = 0; $i < $count; $i++) {
									$map = array_slice($result, 0, 2); // Datensatz einer Map extrahieren
									$map[0] = substr($map[0], -5, 5); // Mapname ausschneiden
									$result = array_slice($result, 2); // Restliche Datenstze zurckschreiben
									
									if (isset($this->mapnames[$this->game][$map[0]])) {
										$this->maplist .= '<option value="'.$i.'">'.$this->mapnames[$this->game][$map[0]].' ('.$map[0].') - '.$map[1].'</option>';
									} else {
										$this->maplist .= '<option value="'.$i.'">'.$map[0].'</option>';
									}
									
								}
								
							}
						}
					}
					
					
					/* SCORETABLE */
					if (true) {
						// Werte fr die Sortierung vorbereiten
						function sort_value($value) {
							$value = round($value, 2);							// Auf zwei Nachkommastellen runden
							$value = $value * 100;								// Die Nachkommastellen entfernen
							$value = $value + 1000000;							// Addieren um negative Werte in positive Werte umzuformen
							$value = str_pad($value, 16, "0", STR_PAD_LEFT);	// Den Wert nach links mit Nullen auffllen
							return $value;
						}
						
						$this->tpl->load_phrases("admin", true);
						$this->tpl->load_phrases("_serv", true);
						
						$dir     = $admin ? "../templates/" : "";
						$show    = $admin ? "" : "display: none";
						
						$id        = $this->id;
						$round     = $this->round;
						$players   = $this->player;
						$team1     = $this->game == "bf3" ? $this->tpl->phrase("us") :  $this->tpl->phrase("away");
						$team2     = $this->game == "bf3" ? $this->tpl->phrase("ru") :  $this->tpl->phrase("home");
						$roundinfo = $this->game == "bf3" ? $this->tpl->phrase("round").' '.$this->round['currentRound'].'/'.$this->round['maxRounds'] : "";
						
						$test    = Array();
						
						eval ("\$html = \"".$this->tpl->template("scoretable", "html", $dir)."\";");  
						$html = phpQuery::newDocumentHTML($html);
						
						// Spieler in die Scoretable einfgen
						foreach($players as $player) {
							$test[] = $player['team'];
							
							// KDR
							$k = $player['kills'] == 0 ? 1 : $player['kills'];
							$d = $player['deaths'] == 0 ? 1 : $player['deaths'];
							$kdr = round(($k/$d), 1);
							
							if ($this->game == "bf3") {
								pq('[team='.$player['team'].'] tbody')->append('
									<tr>
										<td sort="'.$player['squad'].'"><span>'.$player['squad'].'</span></td>
										<td sort="'.$player['name'].'"><a href="http://battlelog.battlefield.com/bf3/user/'.$player['name'].'" target="_blank">'.$player['name'].'</a></td>
										<td sort="'.sort_value($player['kills']).'">'.$player['kills'].'</td>
										<td sort="'.sort_value($player['deaths']).'">'.$player['deaths'].'</td>
										<td sort="'.sort_value($kdr).'">'.$kdr.'</td>
										<td sort="'.sort_value($player['score']).'">'.$player['score'].'</td>
										<td sort="'.$player['name'].'" style="'.$show.'">
											<a href="#" class="button" action="say"  name="'.$player['name'].'">say</a>
											<a href="#" class="button" action="yell" name="'.$player['name'].'">yell</a>
											<a href="#" class="button" action="move" name="'.$player['name'].'">move</a>
											<a href="#" class="button" action="kill" name="'.$player['name'].'">kill</a>
											<a href="#" class="button" action="kick" name="'.$player['name'].'">kick</a>
										</td>
									</tr>
								');
							} else {
								pq('[team='.$player['team'].'] tbody')->append('
									<tr>
										<td sort="'.$player['squad'].'"><span>'.$player['squad'].'</span></td>
										<td sort="'.$player['name'].'"><a href="http://battlelog.medalofhonor.com/mohw/user/'.$player['name'].'" target="_blank">'.$player['name'].'</a></td>
										<td sort="'.sort_value($player['kills']).'">'.$player['kills'].'</td>
										<td sort="'.sort_value($player['deaths']).'">'.$player['deaths'].'</td>
										<td sort="'.sort_value($kdr).'">'.$kdr.'</td>
										<td sort="'.sort_value($player['score']).'">'.$player['score'].'</td>
										<td sort="'.$player['name'].'" style="'.$show.'">
											<a href="#" class="button" action="kill" name="'.$player['name'].'">kill</a>
											<a href="#" class="button" action="kick" name="'.$player['name'].'">kick</a>
										</td>
									</tr>
								');
							}
						}
						
						// Dummy in Scoretable einfgen, wenn kein Spieler im Team ist
						for ($i = 1; $i <= 2; $i++) {
							if (!in_array($i, $test)) {
								pq('[team='.$i.'] tbody')->append('
									<tr>
										<td sort="0"> </td>
										<td sort="0">'.$this->tpl->phrase("rcon_no_player").'</td>
										<td sort="0"> </td>
										<td sort="0"> </td>
										<td sort="0"> </td>
										<td sort="0"> </td>
										<td sort="0" style="'.$show.'"> </td>
									</tr>
								');
							}
						}
						
						$this->score = $html->__toString();
					}
					
					
					/* SONSTIGE */
					$mapcode = $this->rcon->getCurrentMap();
					$mapname = $this->mapnames[$this->game][$mapcode];
					
					$this->mapname  = $mapname.' ('.$mapcode.')';
					$this->playmode = $this->rcon->getCurrentPlaymode();
				}
			}
		}
		
		
		/* Fhrt ein RCON-Kommando aus */
		public function command($command, $mapindex, $name, $message) {
			$this->rcon = new BF3Conn($this->ip, $this->port);
			$this->rcon->loginSecure($this->password);
			
			switch ($command) {
				case "restartround":  $this->rcon->_clientRequest("mapList.restartRound"); break;
				case "nextround":     $this->rcon->_clientRequest("mapList.runNextRound"); break;
				case "setasnextmap":  $this->rcon->_clientRequest("mapList.setNextMapIndex ".$mapindex); break;
				case "runthismap":    $this->rcon->_clientRequest("mapList.setNextMapIndex ".$mapindex); $this->rcon->_clientRequest("mapList.runNextRound"); break;
				
				case "kick":          $this->rcon->adminKickPlayerWithReason($name, $message); break;
				case "kill":          $this->rcon->adminKillPlayer($name); break;
				case "move":          $this->rcon->adminMovePlayerSwitchTeam($name, true); break;
				
				case "say":           $this->rcon->adminSayMessageToPlayer($name, $message); break;
				case "sayus":         $this->rcon->adminSayMessageToTeam(1, $message); break;
				case "sayru":         $this->rcon->adminSayMessageToTeam(2, $message); break;
				case "sayall":        $this->rcon->adminSayMessageToAll($message); break;
				
				case "yell":          $this->rcon->adminYellMessage($message, $name); break;
				case "yellall":       $this->rcon->adminYellMessage($message); break;
				case "yellus":        $this->rcon->adminYellMessageToTeam($message, 1); break;
				case "yellru":        $this->rcon->adminYellMessageToTeam($message, 2); break;
			}
		}
		
		
		/* Ldt die Daten des Servers von der PStats-API */
		public function loadAPI($id, $platform="pc", $game="bf3") {
			
			// Pusht einen Serverupdate
			/*
			$now = time();
			$postdata = Array(
				'request' =>'update',
				'wsrvd'   => 'true',
				'reqid'   => $now.':0'
			);
			$postdata = json_encode($postdata);
			$postdata = 'jsonpost='.urlencode($postdata);
			$c=curl_init($url.'/server/'.$platform.'_'.$id);
			
			$url = $this->urls['url'][$game];
			
			curl_setopt($c,CURLOPT_HEADER,false); curl_setopt($c,CURLOPT_POST,true); curl_setopt($c,CURLOPT_USERAGENT,'BF3StatsAPI/0.1'); curl_setopt($c,CURLOPT_HTTPHEADER,array('Expect:')); curl_setopt($c,CURLOPT_RETURNTRANSFER,true); curl_setopt($c,CURLOPT_POSTFIELDS,$postdata);
			$result = json_decode(curl_exec($c));
			$statuscode = curl_getinfo($c,CURLINFO_HTTP_CODE);
			curl_close($c);
			*/
			
			
			// Abruf der Serverdaten
			$postdata = Array( 'id' => $id );
			$url = $this->urls['api'][$game];
			$c=curl_init($url.$platform.'/server/');
			curl_setopt($c,CURLOPT_HEADER,false); curl_setopt($c,CURLOPT_POST,true); curl_setopt($c,CURLOPT_USERAGENT,'BF3StatsAPI/0.1'); curl_setopt($c,CURLOPT_HTTPHEADER,array('Expect:')); curl_setopt($c,CURLOPT_RETURNTRANSFER,true); curl_setopt($c,CURLOPT_POSTFIELDS,$postdata);
			$result = json_decode(curl_exec($c));
			$statuscode = curl_getinfo($c,CURLINFO_HTTP_CODE);
			curl_close($c);
			
			// Verbindung zu BF3Stats ist fehlgeschlagen
			if ($statuscode != 200 OR !is_object($result)) {
				$this->error = "no_connection";
				$this->message = $result;
				
				return false;
			}
			
			// Server kann bei BF3Stats nicht gefunden werden
			else if ($result->status != "ok" AND $result->status != "found") {
				$error = isset($result->error) ? $result->status.'_'.$result->error : $result->status;
				
				$this->error = "not_found";
				$this->message = $error;
				
				return false;
			}
			
			// Server wurde gefunden
			else {
				$this->error = false;
				$this->info  = $result->srv;
				return true;
			}
		}
		
		
		/* Erweitert die Daten die von der BF3Stats-API geliefert werden */
		private function enhanceData() {
			if ($this->platform == "pc") {
				$url = $this->urls['bl'][$this->game];
				
				$c=curl_init($url.$this->serverid);
				curl_setopt($c,CURLOPT_HEADER,false); curl_setopt($c,CURLOPT_HTTPHEADER,array('X-AjaxNavigation: 1')); curl_setopt($c,CURLOPT_RETURNTRANSFER,true);
				$result = json_decode(curl_exec($c));
				curl_close($c);
				
				// Die Verbindung wurde hergestellt
				if ( $result !== NULL  AND isset($result->context->server) AND is_object($result->context->server)) {
						$info = $result->context->server;
						
						$this->info->players   = $info->numPlayers;
						$this->info->slots     = $info->maxPlayers;
						$this->info->map_name  = $this->mapnames[$this->game][$info->map];
						$this->info->mode_name = $this->gamemodes[$this->game][$info->mapMode];
						$this->info->map_image = strtolower($info->map);
				}
			}
			
			// Fehlende Daten bei Konsolen nachtragen
			else {
				$this->info->pb = 1;
				$this->info->region = "n/a";
			}
		}
		
		
		/* Speichert oder ldt die Daten des Servers im/vom Cache */
		private function cache($option) {
			if ($option == "save") {
				// Objekt in JSON-String konvertieren und Slashes maskieren
				$info = json_encode($this->info);
				if (get_magic_quotes_gpc()) $info = stripslashes($info);
				if (function_exists("mysql_real_escape_string")) $info = mysql_real_escape_string($info); else $info = addslashes($info);
				
				// Serverinfos in Datenbank speichern
				$this->sql->query('
					REPLACE INTO '.$this->sql->prefix.'cache (
						`id`,
						`time`,
						`value`
					) VALUES (
						"'.$this->id.'",
						"'.time().'",
						"'.$info.'"
					)
				');
			}
			
			if ($option == "load") {
				$this->sql->query('SELECT * FROM '.$this->sql->prefix.'cache WHERE `id` = "'.$this->id.'"');
				$info = $this->sql->fetchRow("value");
				
				$this->info = json_decode($info);
				$this->error = false;
			}
		}
		
	}
?>